import { FastifyPluginAsync, FastifyReply, FastifyRequest } from 'fastify'
import fastifyPlugin from 'fastify-plugin'

declare module 'fastify' {
  interface FastifyRequest {
    roles?: string[]
    user?: any
    token?: string
  }
  interface FastifyInstance {
    apiAuth: (request: FastifyRequest, reply: FastifyReply) => {}
  }
}

export interface ApiAuthOptions {
  secret: {
    private: string
    public: string
  }
  expiresIn: string
}

const apiAuthPlugin: FastifyPluginAsync<ApiAuthOptions> = async function (fastify, opts) {
  fastify.register(require('@fastify/jwt'), {
    secret: opts.secret,
    sign: { expiresIn: opts.expiresIn, algorithm: 'EdDSA' },
  })
  // 只有路由配置的role为anon才不需要过滤
  fastify.decorate('apiAuth', async function (request: FastifyRequest, reply: FastifyReply) {
    if (!request.roles || request.roles.indexOf('anon') == -1) {
      // try {
      //   if (!request.token) {
      //     return reply.send({ errcode: 11, errmsg: 'need login' })
      //   }
      //   //@ts-ignore
      //   const data = this.jwt.verify(request.token)
      //   if (!data || !data.id) {
      //     return reply.send({ errcode: 10, errmsg: 'need login' })
      //   }
      // } catch (err) {
      //   return reply.send({ errcode: 401, errmsg: 'need auth' })
      // }
    }
  })
}

export default fastifyPlugin(apiAuthPlugin, '4.x')
